"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const serverPluginHmr_1 = require("./serverPluginHmr");
const hash_sum_1 = __importDefault(require("hash-sum"));
const config_1 = require("./config");
exports.cssPlugin = ({ root, app, watcher, resolver }) => {
    app.use(async (ctx, next) => {
        await next();
        // handle .css imports
        if (ctx.path.endsWith('.css') &&
            // note ctx.body could be null if upstream set status to 304
            ctx.body) {
            if (utils_1.isImportRequest(ctx) &&
                // skip raw requests
                !ctx.query.raw) {
                // we rewrite it to JS that injects a <style> tag pointing to the same url
                // but with a `?raw` query which returns the actual css
                ctx.type = 'js';
                const id = JSON.stringify(hash_sum_1.default(ctx.path));
                const rawPath = JSON.stringify(ctx.path + '?raw');
                ctx.body = `
  import { updateStyle } from "${serverPluginHmr_1.hmrClientId}"\n
  updateStyle(${id}, ${rawPath})
  `.trim();
            }
            else {
                // plain css request, apply postcss transform
                const postcssConfig = await config_1.loadPostcssConfig(root);
                if (postcssConfig) {
                    const css = await utils_1.readBody(ctx.body);
                    try {
                        const result = await require('postcss')(postcssConfig.plugins).process(css, {
                            ...postcssConfig.options,
                            from: resolver.requestToFile(ctx.path)
                        });
                        ctx.body = result.css;
                    }
                    catch (e) {
                        console.error(`[vite] error applying postcss transforms: `, e);
                    }
                }
            }
        }
    });
    // handle hmr
    watcher.on('change', (file) => {
        if (file.endsWith('.css')) {
            const publicPath = resolver.fileToRequest(file);
            const id = hash_sum_1.default(publicPath);
            watcher.send({
                type: 'style-update',
                id,
                path: publicPath,
                timestamp: Date.now()
            });
        }
    });
};
